@@ -265,7 +265,9 @@ If 'never, always use a viewer instead of replacing.")
     (mh-letter-mode (encrypt . mc-encrypt-message)
 		    (sign . mc-sign-message))
     (news-reply-mode (encrypt . mc-encrypt-message)
-		     (sign . mc-sign-message)))
+		     (sign . mc-sign-message))
+    (message-mode (encrypt . mc-encrypt-message)
+		  (sign . mc-sign-message)))
 
   "Association list (indexed by major mode) of association lists
 (indexed by operation) of functions to call for each major mode.")
